@@ -38,6 +38,10 @@ typedef unsigned long long uint64;
 typedef signed long long int64;
 #endif
 
+#ifndef LIBXMP_CORE_PLAYER
+#define LIBXMP_PAULA_SIMULATOR
+#endif
+
 /* Constants */
 #define PAL_RATE	250.0		/* 1 / (50Hz * 80us)		  */
 #define NTSC_RATE	208.0		/* 1 / (60Hz * 80us)		  */
@@ -49,7 +53,7 @@ typedef signed long long int64;
 /* [Amiga] CPU clock = 1.6 * PCCF = 7.0937892 MHz */
 
 #define DEFAULT_AMPLIFY	1
-#define DEFAULT_MIX	70
+#define DEFAULT_MIX	100
 
 #define MSN(x)		(((x)&0xf0)>>4)
 #define LSN(x)		((x)&0x0f)
@@ -135,7 +139,7 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_S3MLOOP	(1 << 0)	/* S3M loop mode */
 #define QUIRK_ENVFADE	(1 << 1)	/* Fade at end of envelope */
 #define QUIRK_PROTRACK	(1 << 2)	/* Use Protracker-specific quirks */
-#define QUIRK_ST3BUGS	(1 << 4)	/* ST 3 weird global volume effect */
+#define QUIRK_ST3BUGS	(1 << 4)	/* Scream Tracker 3 bug compatibility */
 #define QUIRK_FINEFX	(1 << 5)	/* Enable 0xf/0xe for fine effects */
 #define QUIRK_VSALL	(1 << 6)	/* Volume slides in all frames */
 #define QUIRK_PBALL	(1 << 7)	/* Pitch bending in all frames */
@@ -143,8 +147,8 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_VOLPDN	(1 << 9)	/* Set priority to volume slide down */
 #define QUIRK_UNISLD	(1 << 10)	/* Unified pitch slide/portamento */
 #define QUIRK_ITVPOR	(1 << 11)	/* Disable fine bends in IT vol fx */
-#define QUIRK_LINEAR	(1 << 12)	/* Linear periods */
-#define QUIRK_MODRNG	(1 << 13)	/* Limit periods to MOD range */
+/*#define QUIRK_LINEAR	(1 << 12)*/	/* Linear periods */
+/*#define QUIRK_MODRNG	(1 << 13)*/	/* Limit periods to MOD range */
 #define QUIRK_INSVOL	(1 << 14)	/* Use instrument volume */
 #define QUIRK_VIRTUAL	(1 << 15)	/* Enable virtual channels */
 #define QUIRK_FILTER	(1 << 16)	/* Enable filter */
@@ -159,8 +163,8 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_RTDELAY	(1 << 25)	/* Delay effect retrigs instrument */
 #define QUIRK_FT2BUGS	(1 << 26)	/* FT2 bug compatibility */
 #define QUIRK_MARKER	(1 << 27)	/* Patterns 0xfe and 0xff reserved */
-/*#define QUIRK_S3MPMEM	(1 << 28)*/	/* S3M-style parameter memory */
-/*#define QUIRK_XMFINE	(1 << 29)*/	/* XM-style fine tune */
+#define QUIRK_NOBPM	(1 << 28)	/* Adjust speed only, no BPM */
+#define QUIRK_ARPMEM	(1 << 29)	/* Arpeggio has memory (S3M_ARPEGGIO) */
 #define QUIRK_RSTCHN	(1 << 30)	/* Reset channel on sample end */
 
 #define HAS_QUIRK(x)	(m->quirk & (x))
@@ -168,7 +172,7 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 
 /* Format quirks */
 #define QUIRKS_ST3		(QUIRK_S3MLOOP | QUIRK_VOLPDN | QUIRK_FINEFX | \
-				 QUIRK_S3MRTG  | QUIRK_MARKER)
+				 QUIRK_S3MRTG  | QUIRK_MARKER | QUIRK_RSTCHN )
 #define QUIRKS_FT2		(QUIRK_RTDELAY | QUIRK_FINEFX )
 #define QUIRKS_IT		(QUIRK_S3MLOOP | QUIRK_FINEFX | QUIRK_VIBALL | \
 				 QUIRK_ENVFADE | QUIRK_ITVPOR | QUIRK_KEYOFF | \
@@ -190,6 +194,17 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define MAX_SAMPLE_SIZE		0x10000000
 #define MAX_SAMPLES		1024
 
+#define IS_PLAYER_MODE_MOD()	(m->read_event_type == READ_EVENT_MOD)
+#define IS_PLAYER_MODE_FT2()	(m->read_event_type == READ_EVENT_FT2)
+#define IS_PLAYER_MODE_ST3()	(m->read_event_type == READ_EVENT_ST3)
+#define IS_PLAYER_MODE_IT()	(m->read_event_type == READ_EVENT_IT)
+#define IS_PLAYER_MODE_MED()	(m->read_event_type == READ_EVENT_MED)
+#define IS_PERIOD_MODRNG()	(m->period_type == PERIOD_MODRNG)
+#define IS_PERIOD_LINEAR()	(m->period_type == PERIOD_LINEAR)
+#define IS_PERIOD_CSPD()	(m->period_type == PERIOD_CSPD)
+
+#define IS_AMIGA_MOD()	(IS_PLAYER_MODE_MOD() && IS_PERIOD_MODRNG())
+
 struct ord_data {
 	int speed;
 	int bpm;
@@ -236,21 +251,22 @@ struct module_data {
 #define READ_EVENT_IT	3
 #define READ_EVENT_MED	4
 	int read_event_type;
+#define PERIOD_AMIGA	0
+#define PERIOD_MODRNG	1
+#define PERIOD_LINEAR	2
+#define PERIOD_CSPD	3
+	int period_type;
 	int smpctl;			/* sample control flags */
 	int defpan;			/* default pan setting */
 	struct ord_data xxo_info[XMP_MAX_MOD_LENGTH];
-
 	int num_sequences;
 	struct xmp_sequence seq_data[MAX_SEQUENCES];
-
 	char *instrument_path;
-
 	void *extra;			/* format-specific extra fields */
-
-	const struct synth_info *synth;
-	void *synth_chip;
-
 	char **scan_cnt;		/* scan counters */
+#ifndef LIBXMP_CORE_DISABLE_IT
+	struct xmp_sample *xsmp;	/* sustain loop samples */
+#endif
 };
 
 
@@ -261,8 +277,10 @@ struct player_data {
 	int frame;
 	int speed;
 	int bpm;
+	int mode;
 	int player_flags;
 	int flags;
+
 	double current_time;
 	double frame_time;
 
@@ -329,7 +347,7 @@ struct player_data {
 #ifndef LIBXMP_CORE_PLAYER
 	int st26_speed;			/* For IceTracker speed effect */
 #endif
-
+	int filter;			/* Amiga led filter */
 };
 
 struct mixer_data {
@@ -345,7 +363,7 @@ struct mixer_data {
 	int ticksize;
 	int dtright;		/* anticlick control, right channel */
 	int dtleft;		/* anticlick control, left channel */
-	int pbase;		/* period base */
+	double pbase;		/* period base */
 };
 
 struct context_data {
@@ -364,6 +382,7 @@ int	exclude_match		(char *);
 int	prepare_scan		(struct context_data *);
 int	scan_sequences		(struct context_data *);
 int	get_sequence		(struct context_data *, int);
+int	set_player_mode		(struct context_data *);
 
 int8	read8s			(FILE *, int *err);
 uint8	read8			(FILE *, int *err);
