@@ -6,8 +6,8 @@
 #include "inflate.h"
 #include "crc32.h"
 
-#define read_int_b(x) read32b(x)
-#define read_word(x) read16l(x)
+#define read_int_b(x) read32b(x, NULL)
+#define read_word(x) read16l(x, NULL)
 
 #define ZIP
 #undef DEBUG
@@ -717,7 +717,11 @@ int decompress(FILE *in, struct huffman_t *huffman, struct bitstream_t *bitstrea
       {
         /* bitstream->holding+=(getc(in)<<bitstream->bitptr); */
         /* bitstream->bitptr+=8; */
-        bitstream->holding=getc(in);
+        int x = getc(in);
+        if (x == EOF) {
+          return -1;
+        }
+        bitstream->holding=x;
         bitstream->bitptr=8;
       }
 #ifdef DEBUG
@@ -1014,8 +1018,16 @@ int inflate(FILE *in, FILE *out, uint32 *checksum, int is_zip)
 #endif
 
 if (!is_zip) {
-  CMF=getc(in);
-  FLG=getc(in);
+  int x;
+  if ((x = getc(in)) < 0) {
+    goto err3;
+  }
+  CMF=x;
+
+  if ((x = getc(in)) < 0) {
+    goto err3;
+  }
+  FLG=x;
 
 #ifdef DEBUG
   printf("   CMF: %d\n",CMF);
@@ -1090,7 +1102,11 @@ if (!is_zip) {
 
       for (t=0; t<block_len; t++)
       {
-        huffman.window[huffman.window_ptr++]=getc(in);
+        res = getc(in);
+        if (res < 0) {
+          goto err3;
+        }
+        huffman.window[huffman.window_ptr++]=res;
 
         if (huffman.window_ptr>=WINDOW_SIZE)
         {
