@@ -67,9 +67,10 @@
 } while (0)
 
 
-static void do_toneporta(struct module_data *m,
+static void do_toneporta(struct context_data *ctx,
                                 struct channel_data *xc, int note)
 {
+	struct module_data *m = &ctx->m;
 	struct xmp_instrument *instrument = &m->mod.xxi[xc->ins];
 	int mapped = instrument->map[xc->key].ins;
 	struct xmp_subinstrument *sub;
@@ -82,9 +83,9 @@ static void do_toneporta(struct module_data *m,
 
 	if (note >= 1 && note <= 0x80 && (uint32)xc->ins < m->mod.ins) {
 		note--;
-		xc->porta.target = note_to_period(note + sub->xpo +
+		xc->porta.target = note_to_period(ctx, note + sub->xpo +
 			instrument->map[xc->key_porta].xpo, xc->finetune,
-			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+			xc->per_adj);
 	}
 	xc->porta.dir = xc->period < xc->porta.target ? 1 : -1;
 }
@@ -116,7 +117,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	switch (fxt) {
 	case FX_ARPEGGIO:
 	      fx_arpeggio:
-		if (fxp != 0) {
+		if (!HAS_QUIRK(QUIRK_ARPMEM) || fxp != 0) {
 			xc->arpeggio.val[0] = 0;
 			xc->arpeggio.val[1] = MSN(fxp);
 			xc->arpeggio.val[2] = LSN(fxp);
@@ -126,6 +127,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_S3M_ARPEGGIO:
 		EFFECT_MEMORY(fxp, xc->arpeggio.memory);
 		goto fx_arpeggio;
+
 #ifndef LIBXMP_CORE_PLAYER
 	case FX_OKT_ARP3:
 		if (fxp != 0) {
@@ -222,7 +224,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		if (!IS_VALID_INSTRUMENT(xc->ins))
 			break;
 
-		do_toneporta(m, xc, note);
+		do_toneporta(ctx, xc, note);
 
 		SET(TONEPORTA);
 		break;
@@ -243,7 +245,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_TONE_VSLIDE:	/* Toneporta + vol slide */
 		if (!IS_VALID_INSTRUMENT(xc->ins))
 			break;
-		do_toneporta(m, xc, note);
+		do_toneporta(ctx, xc, note);
 		SET(TONEPORTA);
 		goto fx_volslide;
 	case FX_VIBRA_VSLIDE:	/* Vibrato + vol slide */
@@ -375,6 +377,11 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		fxt = fxp >> 4;
 		fxp &= 0x0f;
 		switch (fxt) {
+		case EX_FILTER:		/* Amiga led filter */
+			if (IS_AMIGA_MOD()) {
+				p->filter = !(fxp & 1);
+			}
+			break;
 		case EX_F_PORTA_UP:	/* Fine portamento up */
 			EFFECT_MEMORY(fxp, xc->fine_porta.up_memory);
 			goto fx_f_porta_up;
@@ -453,8 +460,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_SPEED:		/* Set speed */
+		if (HAS_QUIRK(QUIRK_NOBPM) || p->flags & XMP_FLAGS_VBLANK) {
+			goto fx_s3m_speed;
+		}
+
 		/* speedup.xm needs BPM = 20 */
-		if (p->flags & XMP_FLAGS_VBLANK || fxp < 0x20) {
+		if (fxp < 0x20) {
 			goto fx_s3m_speed;
 		} else {
 			goto fx_s3m_bpm;
@@ -677,7 +688,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		EFFECT_MEMORY(fxp, xc->tremor.memory);
 		xc->tremor.up = MSN(fxp);
 		xc->tremor.down = LSN(fxp);
-		if (m->read_event_type == READ_EVENT_FT2) {
+		if (IS_PLAYER_MODE_FT2()) {
 			xc->tremor.count |= 0x80;
 		} else {
 			if (xc->tremor.up == 0) {
@@ -842,14 +853,14 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 	case FX_PITCH_ADD:
 		SET_PER(TONEPORTA);
-		xc->porta.target = note_to_period(note - 1, xc->finetune, 0, 0)
+		xc->porta.target = note_to_period(ctx, note - 1, xc->finetune, 0)
 			+ fxp;
 		xc->porta.slide = 2;
 		xc->porta.dir = 1;
 		break;
 	case FX_PITCH_SUB:
 		SET_PER(TONEPORTA);
-		xc->porta.target = note_to_period(note - 1, xc->finetune, 0, 0)
+		xc->porta.target = note_to_period(ctx, note - 1, xc->finetune, 0)
 			- fxp;
 		xc->porta.slide = 2;
 		xc->porta.dir = -1;
@@ -974,7 +985,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		if (!IS_VALID_INSTRUMENT(xc->ins))
 			break;
 		SET_PER(TONEPORTA);
-		do_toneporta(m, xc, note);
+		do_toneporta(ctx, xc, note);
 		xc->porta.slide = fxp;
 		if (fxp == 0)
 			RESET_PER(TONEPORTA);
@@ -998,15 +1009,50 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_SPEED_CP:	/* Set speed and ... */
 		if (fxp) {
 			p->speed = fxp;
-#ifndef LIBXMP_CORE_PLAYER
 			p->st26_speed = 0;
-#endif
 		}
 		/* fall through */
 	case FX_PER_CANCEL:	/* Cancel persistent effects */
 		xc->per_flags = 0;
 		break;
+
+	/* 669 effects */
+
+	case FX_669_PORTA_UP:	/* 669 portamento up */
+		SET_PER(PITCHBEND);
+		xc->freq.slide = 80 * fxp;
+		if ((xc->freq.memory = fxp) == 0)
+			RESET_PER(PITCHBEND);
+		break;
+	case FX_669_PORTA_DN:	/* 669 portamento down */
+		SET_PER(PITCHBEND);
+		xc->freq.slide = -80 * fxp;
+		if ((xc->freq.memory = fxp) == 0)
+			RESET_PER(PITCHBEND);
+		break;
+	case FX_669_TPORTA:	/* 669 tone portamento */
+		if (!IS_VALID_INSTRUMENT(xc->ins))
+			break;
+		SET_PER(TONEPORTA);
+		do_toneporta(ctx, xc, note);
+		xc->porta.slide = 40 * fxp;
+		if (fxp == 0)
+			RESET_PER(TONEPORTA);
+		break;
+	case FX_669_FINETUNE:	/* 669 finetune */
+		xc->finetune = 80 * (int8)fxp;
+		break;
+	case FX_669_VIBRATO:	/* 669 vibrato */
+		if (LSN(fxp) != 0) {
+			set_lfo_waveform(&xc->vibrato.lfo, 669);
+			SET_PER(VIBRATO);
+		} else {
+			RESET_PER(VIBRATO);
+		}
+		SET_LFO_NOTZERO(&xc->vibrato.lfo, 669, 1);
+		break;
 #endif
+
 	default:
 #ifndef LIBXMP_CORE_PLAYER
 		extras_process_fx(ctx, xc, chn, note, fxt, fxp, fnum);
