@@ -37,7 +37,6 @@
 #endif
 
 #include <fnmatch.h>
-#include "synth.h"
 
 /*
  * Handle special "module quirks" that can't be detected automatically
@@ -47,6 +46,7 @@
 struct module_quirk {
 	uint8 md5[16];
 	int flags;
+	int mode;
 };
 
 const struct module_quirk mq[] = {
@@ -54,41 +54,59 @@ const struct module_quirk mq[] = {
 	{
 		{ 0x36, 0x6e, 0xc0, 0xfa, 0x96, 0x2a, 0xeb, 0xee,
 	  	  0x03, 0x4a, 0xa2, 0xdb, 0xaa, 0x49, 0xaa, 0xea },
-		XMP_FLAGS_FX9BUG
+		0, XMP_MODE_PROTRACKER
 	},
 
 	/* mod.souvenir of china */
 	{
 		{ 0x93, 0xf1, 0x46, 0xae, 0xb7, 0x58, 0xc3, 0x9d,
 		  0x8b, 0x5f, 0xbc, 0x98, 0xbf, 0x23, 0x7a, 0x43 },
-		XMP_FLAGS_FIXLOOP
+		XMP_FLAGS_FIXLOOP, XMP_MODE_AUTO
 	},
 
 	/* "siedler ii" (added by Daniel Åkerud) */
 	{
 		{ 0x70, 0xaa, 0x03, 0x4d, 0xfb, 0x2f, 0x1f, 0x73,
 		  0xd9, 0xfd, 0xba, 0xfe, 0x13, 0x1b, 0xb7, 0x01 },
-		XMP_FLAGS_VBLANK
+		XMP_FLAGS_VBLANK, XMP_MODE_AUTO
 	},
 
 	/* "Klisje paa klisje" (added by Kjetil Torgrim Homme) */
 	{
 		{ 0xe9, 0x98, 0x01, 0x2c, 0x70, 0x0e, 0xb4, 0x3a,
 		  0xf0, 0x32, 0x17, 0x11, 0x30, 0x58, 0x29, 0xb2 },
-		XMP_FLAGS_VBLANK
+		0, XMP_MODE_NOISETRACKER
 	},
 
+#if 0
+	/* -- Already covered by Noisetracker fingerprinting -- */
+
 	/* Another version of Klisje paa klisje sent by Steve Fernandez */
 	{
 		{ 0x12, 0x19, 0x1c, 0x90, 0x41, 0xe3, 0xfd, 0x70,
 		  0xb7, 0xe6, 0xb3, 0x94, 0x8b, 0x21, 0x07, 0x63 },
 		XMP_FLAGS_VBLANK
 	},
+#endif
+
+	/* "((((( nebulos )))))" sent by Tero Auvinen (AMP version) */
+	{
+		{ 0x51, 0x6e, 0x8d, 0xcc, 0x35, 0x7d, 0x50, 0xde,
+		  0xa9, 0x85, 0xbe, 0xbf, 0x90, 0x2e, 0x42, 0xdc },
+		0, XMP_MODE_NOISETRACKER
+	},
+
+	/* Purple Motion's Sundance.mod, Music Channel BBS edit */
+	{
+		{ 0x5d, 0x3e, 0x1e, 0x08, 0x28, 0x52, 0x12, 0xc7,
+		  0x17, 0x64, 0x95, 0x75, 0x98, 0xe6, 0x95, 0xc1 },
+		0, XMP_MODE_ST3
+	},
 
 	{
 		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
-		0
+		0, 0
 	}
 };
 
@@ -98,9 +116,10 @@ static void module_quirks(struct context_data *ctx)
 	struct module_data *m = &ctx->m;
 	int i;
 
-	for (i = 0; mq[i].flags != 0; i++) {
+	for (i = 0; mq[i].flags != 0 || mq[i].mode != 0; i++) {
 		if (!memcmp(m->md5, mq[i].md5, 16)) {
 			p->flags |= mq[i].flags;
+			p->mode = mq[i].mode;
 		}
 	}
 }
@@ -183,6 +202,7 @@ void load_prologue(struct context_data *ctx)
 	m->vol_table = NULL;
 	m->quirk = 0;
 	m->read_event_type = READ_EVENT_MOD;
+	m->period_type = PERIOD_AMIGA;
 	m->comment = NULL;
 	m->scan_cnt = NULL;
 
@@ -198,9 +218,12 @@ void load_prologue(struct context_data *ctx)
     	m->mod.rst = 0;
 
 #ifndef LIBXMP_CORE_PLAYER
-	m->synth = &synth_null;
 	m->extra = NULL;
 #endif
+#ifndef LIBXMP_CORE_DISABLE_IT
+	m->xsmp = NULL;
+#endif
+
 	m->time_factor = DEFAULT_TIME_FACTOR;
 
 	for (i = 0; i < 64; i++) {
@@ -265,10 +288,13 @@ void load_epilogue(struct context_data *ctx)
 		check_envelope(&mod->xxi[i].pei);
 	}
 
+	p->filter = 0;
+	p->mode = XMP_MODE_AUTO;
 	p->flags = p->player_flags;
 #ifndef LIBXMP_CORE_PLAYER
 	module_quirks(ctx);
 #endif
+	set_player_mode(ctx);
 }
 
 int prepare_scan(struct context_data *ctx)
@@ -311,3 +337,69 @@ int prepare_scan(struct context_data *ctx)
  
 	return 0;
 }
+
+/* Process player personality flags */
+int set_player_mode(struct context_data *ctx)
+{
+	struct player_data *p = &ctx->p;
+	struct module_data *m = &ctx->m;
+	int q;
+
+	switch (p->mode) {
+	case XMP_MODE_AUTO:
+		break;
+	case XMP_MODE_MOD:
+		m->quirk = 0;
+		m->read_event_type = READ_EVENT_MOD;
+		m->period_type = PERIOD_AMIGA;
+		break;
+	case XMP_MODE_NOISETRACKER:
+		m->quirk = QUIRK_NOBPM;
+		m->read_event_type = READ_EVENT_MOD;
+		m->period_type = PERIOD_MODRNG;
+		break;
+	case XMP_MODE_PROTRACKER:
+		m->quirk = QUIRK_PROTRACK;
+		m->read_event_type = READ_EVENT_MOD;
+		m->period_type = PERIOD_MODRNG;
+		break;
+	case XMP_MODE_S3M:
+		q = m->quirk & (QUIRK_VSALL | QUIRK_ARPMEM);
+		m->quirk = QUIRKS_ST3 | q;
+		m->read_event_type = READ_EVENT_ST3;
+		break;
+	case XMP_MODE_ST3:
+		q = m->quirk & (QUIRK_VSALL | QUIRK_ARPMEM);
+		m->quirk = QUIRKS_ST3 | QUIRK_ST3BUGS | q;
+		m->read_event_type = READ_EVENT_ST3;
+		break;
+	case XMP_MODE_ST3GUS:
+		q = m->quirk & (QUIRK_VSALL | QUIRK_ARPMEM);
+		m->quirk = QUIRKS_ST3 | QUIRK_ST3BUGS | q;
+		m->quirk &= ~QUIRK_RSTCHN;
+		m->read_event_type = READ_EVENT_ST3;
+		break;
+	case XMP_MODE_XM:
+		m->quirk = QUIRKS_FT2;
+		m->read_event_type = READ_EVENT_FT2;
+		break;
+	case XMP_MODE_FT2:
+		m->quirk = QUIRKS_FT2 | QUIRK_FT2BUGS;
+		m->read_event_type = READ_EVENT_FT2;
+		break;
+	case XMP_MODE_IT:
+		m->quirk = QUIRKS_IT | QUIRK_VIBHALF | QUIRK_VIBINV;
+		m->read_event_type = READ_EVENT_IT;
+		break;
+	case XMP_MODE_ITSMP:
+		m->quirk = QUIRKS_IT | QUIRK_VIBHALF | QUIRK_VIBINV;
+		m->quirk &= ~(QUIRK_VIRTUAL | QUIRK_RSTCHN);
+		m->read_event_type = READ_EVENT_IT;
+		break;
+	default:
+		return -1;
+	}
+
+	return 0;
+}
+
