@@ -174,7 +174,7 @@ static int chip_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 	}
 
-	m->quirk |= QUIRK_MODRNG;
+	m->period_type = PERIOD_MODRNG;
 
 	/* Load samples */
 
