@@ -186,7 +186,7 @@ static int ice_load(struct module_data *m, HIO_HANDLE * f, const int start)
 		}
 	}
 
-	m->quirk |= QUIRK_MODRNG;
+	m->period_type = PERIOD_MODRNG;
 
 	/* Read samples */
 
