@@ -45,7 +45,6 @@
 #include "player.h"
 #include "mixer.h"
 #ifndef LIBXMP_CORE_PLAYER
-#include "synth.h"
 #include "extras.h"
 #endif
 
@@ -109,6 +108,49 @@ static void update_invloop(struct module_data *m, struct channel_data *xc)
 
 #endif
 
+/*
+ * From OpenMPT Arpeggio.xm test:
+ *
+ * "[FT2] Arpeggio behavior is very weird with more than 16 ticks per row. This
+ *  comes from the fact that Fasttracker 2 uses a LUT for computing the arpeggio
+ *  note (instead of doing something like tick%3 or similar). The LUT only has
+ *  16 entries, so when there are more than 16 ticks, it reads beyond array
+ *  boundaries. The vibrato table happens to be stored right after arpeggio
+ *  table. The tables look like this in memory:
+ *
+ *    ArpTab: 0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0
+ *    VibTab: 0,24,49,74,97,120,141,161,180,197,...
+ *
+ *  All values except for the first in the vibrato table are greater than 1, so
+ *  they trigger the third arpeggio note. Keep in mind that Fasttracker 2 counts
+ *  downwards, so the table has to be read from back to front, i.e. at 16 ticks
+ *  per row, the 16th entry in the LUT is the first to be read. This is also the
+ *  reason why Arpeggio is played 'backwards' in Fasttracker 2."
+ */
+static int ft2_arpeggio(struct context_data *ctx, struct channel_data *xc)
+{
+	struct player_data *p = &ctx->p;
+	int i;
+
+	if (xc->arpeggio.val[1] == 0 && xc->arpeggio.val[2] == 0) {
+		return 0;
+	}
+
+	if (p->frame == 0) {
+		return 0;
+	}
+
+	i = p->speed - (p->frame % p->speed);
+
+	if (i == 16) {
+		return 0;
+	} else if (i > 16) {
+		return xc->arpeggio.val[2];
+	}
+
+	return xc->arpeggio.val[i % 3];
+}
+
 static int is_first_frame(struct context_data *ctx)
 {
 	struct player_data *p = &ctx->p;
@@ -136,8 +178,6 @@ static void reset_channels(struct context_data *ctx)
 	int i;
 
 #ifndef LIBXMP_CORE_PLAYER
-	m->synth->reset(ctx);
-
 	for (i = 0; i < p->virt.virt_channels; i++) {
 		void *extra;
 
@@ -240,7 +280,7 @@ static int check_delay(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	if (HAS_QUIRK(QUIRK_RTDELAY)) {
-		if (e->vol == 0 && e->f2t == 0 && e->ins == 0 && e->note == 0)
+		if (e->vol == 0 && e->f2t == 0 && e->ins == 0 && e->note != XMP_KEY_OFF)
 			xc->delayed_event.vol = xc->volume + 1;
 		if (e->note == 0)
 			xc->delayed_event.note = xc->key + 1;
@@ -295,7 +335,7 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 #endif
 			}
 		} else {
-			if (m->read_event_type == READ_EVENT_IT) {
+			if (IS_PLAYER_MODE_IT()) {
 				/* Reset flags. See SlideDelay.it */
 				p->xc_data[chn].flags = 0;
 			}
@@ -389,7 +429,6 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	struct xmp_instrument *instrument;
 	int finalvol;
 	uint16 vol_envelope;
-	int gvol;
 	int fade = 0;
 
 	instrument = get_instrument(ctx, xc->ins);
@@ -452,14 +491,14 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 
 	if (!TEST_PER(VENV_PAUSE)) {
 		xc->v_idx = update_envelope(&instrument->aei, xc->v_idx,
-					DOENV_RELEASE, TEST(KEY_OFF),
-					m->read_event_type == READ_EVENT_IT);
+			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
 	}
 
 	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64);
 	if (check_envelope_end(&instrument->aei, xc->v_idx)) {
-		if (vol_envelope == 0)
+		if (vol_envelope == 0) {
 			SET_NOTE(NOTE_END);
+		}
 		SET_NOTE(NOTE_ENV_END);
 	}
 
@@ -475,12 +514,16 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	finalvol = xc->volume;
 #endif
 
-	if (m->read_event_type == READ_EVENT_IT) {
+	if (IS_PLAYER_MODE_IT()) {
 		finalvol = xc->volume * (100 - xc->rvv) / 100;
 	}
 
 	if (TEST(TREMOLO)) {
-		finalvol += get_lfo(ctx, &xc->tremolo.lfo, 0) / (1 << 6);
+		/* OpenMPT VibratoReset.mod */
+		if (!is_first_frame(ctx) || !HAS_QUIRK(QUIRK_PROTRACK)) {
+			finalvol += get_lfo(ctx, &xc->tremolo.lfo, 0) / (1 << 6);
+		}
+
 		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
 			update_lfo(&xc->tremolo.lfo);
 		}
@@ -490,14 +533,8 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 
 	finalvol = (finalvol * xc->fadeout) >> 6;	/* 16 bit output */
 
-	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
-		gvol = 0x40;
-	} else {
-		gvol = p->gvol;
-	}
-
-	finalvol = (uint32)(vol_envelope * gvol * xc->mastervol / m->gvolbase *
-				((int)finalvol * 0x40 / m->volbase)) >> 18;
+	finalvol = (uint32)(vol_envelope * p->gvol * xc->mastervol /
+		m->gvolbase * ((int)finalvol * 0x40 / m->volbase)) >> 18;
 
 	/* Apply channel volume */
 	finalvol = finalvol * get_channel_vol(ctx, chn) / 100;
@@ -515,7 +552,7 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 		finalvol = (finalvol * instrument->vol * xc->gvl) >> 12;
 	}
 
-	if (m->read_event_type == READ_EVENT_FT2) {
+	if (IS_PLAYER_MODE_FT2()) {
 		finalvol = tremor_ft2(ctx, chn, finalvol);
 	} else {
 		finalvol = tremor_s3m(ctx, chn, finalvol);
@@ -558,8 +595,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 
 	if (!TEST_PER(FENV_PAUSE)) {
 		xc->f_idx = update_envelope(&instrument->fei, xc->f_idx,
-					DOENV_RELEASE, TEST(KEY_OFF),
-					m->read_event_type == READ_EVENT_IT);
+			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
 	}
 	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
 
@@ -569,8 +605,8 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	if (TEST(NOTE_SLIDE)) {
 		if (xc->noteslide.count == 0) {
 			xc->note += xc->noteslide.slide;
-			xc->period = note_to_period(xc->note, xc->finetune,
-					HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+			xc->period = note_to_period(ctx, xc->note, xc->finetune,
+					xc->per_adj);
 			xc->noteslide.count = xc->noteslide.speed;
 		}
 		xc->noteslide.count--;
@@ -591,13 +627,16 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 
 	/* Vibrato */
 	if (TEST(VIBRATO) || TEST_PER(VIBRATO)) {
-		int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
-		int vib = get_lfo(ctx, &xc->vibrato.lfo, 1) / (1 << shift);
+		/* OpenMPT VibratoReset.mod */
+		if (!is_first_frame(ctx) || !HAS_QUIRK(QUIRK_PROTRACK)) {
+			int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
+			int vib = get_lfo(ctx, &xc->vibrato.lfo, 1) / (1 << shift);
 
-		if (HAS_QUIRK(QUIRK_VIBINV)) {
-			vibrato -= vib;
-		} else {
-			vibrato += vib;
+			if (HAS_QUIRK(QUIRK_VIBINV)) {
+				vibrato -= vib;
+			} else {
+				vibrato += vib;
+			}
 		}
 
 		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
@@ -610,13 +649,21 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	period += extras_get_period(ctx, xc);
 #endif
 
+	/* Sanity check */
+	if (period < 0.1) {
+		period = 0.1;
+	} 
+
 	/* Arpeggio */
 
-	arp = xc->arpeggio.val[xc->arpeggio.count];
+	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
+		arp = ft2_arpeggio(ctx, xc);
+	} else {
+		arp = xc->arpeggio.val[xc->arpeggio.count];
+	}
 
 	/* Pitch bend */
 
-
  	/* From OpenMPT PeriodLimit.s3m:
 	 * "ScreamTracker 3 limits the final output period to be at least 64,
 	 *  i.e. when playing a note that is too high or when sliding the
@@ -628,13 +675,13 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	 * A6 value because we compute periods in floating point.
 	 */
 	if (HAS_QUIRK(QUIRK_ST3BUGS)) {
-		if (period < 16.236899) {	/* A6 */
-			period = 16.236899;
+		if (period < 16.239270) {	/* A6 */
+			period = 16.239270;
 		}
 	}
 
-	linear_bend = period_to_bend(period + vibrato, xc->note,
-			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+	linear_bend = period_to_bend(ctx, period + vibrato, xc->note,
+							xc->per_adj);
 
 	if (TEST_NOTE(NOTE_GLISSANDO) && TEST(TONEPORTA)) {
 		if (linear_bend > 0) {
@@ -644,10 +691,21 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 		}
 	}
 
-	if (HAS_QUIRK(QUIRK_FT2BUGS) && arp) {
-		linear_bend = linear_bend / 12800 * 12800;
-	}
+	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
+		if  (arp) {
+			/* OpenMPT ArpSlide.xm */
+			linear_bend = linear_bend / 12800 * 12800 +
+							xc->finetune * 100;
 
+			/* OpenMPT ArpeggioClamp.xm */
+			if (xc->note + arp > 107) {
+				if (p->speed - (p->frame % p->speed) > 0) {
+					arp = 108 - xc->note;
+				}
+			}
+		}
+	}
+	
 	/* Envelope */
 
 	if (xc->f_idx >= 0 && (~instrument->fei.flg & XMP_ENVELOPE_FLT)) {
@@ -657,23 +715,16 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 		linear_bend += frq_envelope << 7;
 	}
 
-	/* OpenMPT ArpeggioClamp.xm */
-	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
-		if (xc->note + arp > 107) {
-			if (xc->arpeggio.count > 0) {
-				arp = 108 - xc->note;
-			}
-		}
-	}
-	
+	/* Arpeggio */
+
 	if (arp != 0) {
 		linear_bend += (100 << 7) * arp;
 
 		/* OpenMPT ArpWrapAround.mod */
 		if (HAS_QUIRK(QUIRK_PROTRACK)) {
-			if (xc->note + arp > 84) {
+			if (xc->note + arp > MAX_NOTE_MOD + 1) {
 				linear_bend -= 12800 * (3 * 12);
-			} else if (xc->note + arp > 83) {
+			} else if (xc->note + arp > MAX_NOTE_MOD) {
 				virt_setvol(ctx, chn, 0);
 			}
 		}
@@ -684,19 +735,21 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	linear_bend += extras_get_linear_bend(ctx, xc);
 #endif
 
+	period = note_to_period_mix(xc->note, linear_bend);
+	virt_setperiod(ctx, chn, period);
+
 	/* For xmp_get_frame_info() */
 	xc->info_pitchbend = linear_bend >> 7;
-	xc->info_period = note_to_period_mix(xc->note, linear_bend);
+	xc->info_period = period * 4096;
 
-	if (HAS_QUIRK(QUIRK_MODRNG)) {
+	if (IS_PERIOD_MODRNG()) {
 		CLAMP(xc->info_period,
-			note_to_period(83, xc->finetune, 0, 0) * 4096,
-			note_to_period(48, xc->finetune, 0, 0) * 4096);
-	} else if (xc->info_period <  (1 << 12)) {
+			note_to_period(ctx, MAX_NOTE_MOD, xc->finetune, 0) * 4096,
+			note_to_period(ctx, MIN_NOTE_MOD, xc->finetune, 0) * 4096);
+	} else if (xc->info_period < (1 << 12)) {
 		xc->info_period = (1 << 12);
 	}
 
-	virt_setbend(ctx, chn, linear_bend);
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
@@ -742,13 +795,13 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	struct xmp_instrument *instrument;
 	int finalpan, panbrello = 0;
 	int pan_envelope;
+	int channel_pan;
 
 	instrument = get_instrument(ctx, xc->ins);
 
 	if (!TEST_PER(PENV_PAUSE)) {
 		xc->p_idx = update_envelope(&instrument->pei, xc->p_idx,
-					DOENV_RELEASE, TEST(KEY_OFF),
-					m->read_event_type == READ_EVENT_IT);
+			DOENV_RELEASE, TEST(KEY_OFF), IS_PLAYER_MODE_IT());
 	}
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 
@@ -761,10 +814,23 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	}
 #endif
 
-	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
+	channel_pan = xc->pan.val;
+
+#if 0
+#ifdef LIBXMP_PAULA_SIMULATOR
+	/* Always use 100% pan separation in Amiga mode */
+	if (p->flags & XMP_FLAGS_A500) {
+		if (IS_AMIGA_MOD()) {
+			channel_pan = channel_pan < 0x80 ? 0 : 0xff;
+		}
+	}
+#endif
+#endif
+
+	finalpan = channel_pan + panbrello + (pan_envelope - 32) *
 				(128 - abs(xc->pan.val - 128)) / 32;
 
-	if (m->read_event_type == READ_EVENT_IT) {
+	if (IS_PLAYER_MODE_IT()) {
 		finalpan = finalpan + xc->rpv * 4;
 	}
 
@@ -918,18 +984,21 @@ static void update_frequency(struct context_data *ctx, int chn)
 #ifndef LIBXMP_CORE_PLAYER
 		if (TEST(FINE_NSLIDE)) {
 			xc->note += xc->noteslide.fslide;
-			xc->period = note_to_period(xc->note,
-				xc->finetune, HAS_QUIRK(QUIRK_LINEAR),
-				xc->per_adj);
+			xc->period = note_to_period(ctx, xc->note,
+				xc->finetune, xc->per_adj);
 		}
 #endif
 	}
 
-	if (HAS_QUIRK(QUIRK_LINEAR)) {
+	switch (m->period_type) {
+	case PERIOD_LINEAR:
 		CLAMP(xc->period, MIN_PERIOD_L, MAX_PERIOD_L);
-	} else if (HAS_QUIRK(QUIRK_MODRNG)) {
-		CLAMP(xc->period, note_to_period(83, xc->finetune, 0, 0),
-				  note_to_period(48, xc->finetune, 0, 0));
+		break;
+	case PERIOD_MODRNG:
+		CLAMP(xc->period,
+			note_to_period(ctx, MAX_NOTE_MOD, xc->finetune, 0),
+			note_to_period(ctx, MIN_NOTE_MOD, xc->finetune, 0));
+		break;
 	}
 
 	xc->arpeggio.count++;
@@ -1037,6 +1106,8 @@ static void play_channel(struct context_data *ctx, int chn)
 			SET_NOTE(NOTE_RELEASE);
 	}
 
+	virt_release(ctx, chn, TEST_NOTE(NOTE_RELEASE));
+
 	process_volume(ctx, chn, act);
 	process_frequency(ctx, chn, act);
 	process_pan(ctx, chn, act);
@@ -1217,9 +1288,6 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
 	struct player_data *p = &ctx->p;
-#ifndef LIBXMP_CORE_PLAYER
-	struct mixer_data *s = &ctx->s;
-#endif
 	struct smix_data *smix = &ctx->smix;
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
@@ -1306,13 +1374,6 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 		if (new_channel_extras(ctx, xc) < 0)
 			goto err2;
 	}
-
-	if (m->synth->init(ctx, s->freq) < 0) {
-		ret = -XMP_ERROR_INTERNAL;
-		goto err2;
-	}
-
-	m->synth->reset(ctx);
 #endif
 	reset_channels(ctx);
 
@@ -1523,7 +1584,6 @@ void xmp_end_player(xmp_context opaque)
 	struct player_data *p = &ctx->p;
 	struct flow_control *f = &p->flow;
 #ifndef LIBXMP_CORE_PLAYER
-	struct module_data *m = &ctx->m;
 	struct channel_data *xc;
 	int i;
 #endif
@@ -1542,9 +1602,6 @@ void xmp_end_player(xmp_context opaque)
 #endif
 
 	virt_off(ctx);
-#ifndef LIBXMP_CORE_PLAYER
-	m->synth->deinit(ctx);
-#endif
 
 	free(p->xc_data);
 	free(f->loop);
