@@ -177,8 +177,8 @@ static void set_period(struct context_data *ctx, int note,
 	struct module_data *m = &ctx->m;
 
 	if (sub != NULL && note >= 0) {
-		double per = note_to_period(note, xc->finetune,
-				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+		double per = note_to_period(ctx, note, xc->finetune,
+							xc->per_adj);
 
 		if (!HAS_QUIRK(QUIRK_PROTRACK) || (note > 0 && is_toneporta)) {
 			xc->porta.target = per;
@@ -199,16 +199,14 @@ static void set_period_ft2(struct context_data *ctx, int note,
 				struct xmp_subinstrument *sub,
 				struct channel_data *xc, int is_toneporta)
 {
-	struct module_data *m = &ctx->m;
-
 	if (note > 0 && is_toneporta) {
-		xc->porta.target = note_to_period(note, xc->finetune,
-				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+		xc->porta.target = note_to_period(ctx, note, xc->finetune,
+								xc->per_adj);
 	}
 	if (sub != NULL && note >= 0) {
 		if (xc->period < 1 || !is_toneporta) {
-			xc->period = note_to_period(note, xc->finetune,
-				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+			xc->period = note_to_period(ctx, note, xc->finetune,
+								xc->per_adj);
 		}
 	}
 }
@@ -342,8 +340,8 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 
 #ifndef LIBXMP_CORE_PLAYER
 	if (IS_SFX_PITCH(e->fxt)) {
- 		xc->period = note_to_period(note, xc->finetune,
-                                HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+ 		xc->period = note_to_period(ctx, note, xc->finetune,
+                                			xc->per_adj);
 	} else
 #endif
 	{
@@ -466,8 +464,9 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 			/* No note */
 			if (sub != NULL) {
+				int p = mod->xxc[chn].pan - 128;
 				xc->volume = sub->vol;
-				xc->pan.val = sub->pan;
+				xc->pan.val = p + ((sub->pan - 128) * (128 - abs(p))) / 128 + 128;
 				xc->ins_fade = mod->xxi[xc->ins].rls;
 				SET(NEW_VOL);
 			}
@@ -520,8 +519,9 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			/* Current instrument */
 			sub = get_subinstrument(ctx, xc->ins, key - 1);
 			if (sub != NULL) {
+				int p = mod->xxc[chn].pan - 128;
 				xc->volume = sub->vol;
-				xc->pan.val = sub->pan;
+				xc->pan.val = p + ((sub->pan - 128) * (128 - abs(p))) / 128 + 128;
 				xc->ins_fade = mod->xxi[xc->ins].rls;
 			} else {
 				xc->volume = 0;
@@ -911,13 +911,11 @@ static void fix_period(struct context_data *ctx, int chn, struct xmp_subinstrume
 {
 	if (sub->nna == XMP_INST_NNA_CONT) {
 		struct player_data *p = &ctx->p;
-		struct module_data *m = &ctx->m;
 		struct channel_data *xc = &p->xc_data[chn];
 		struct xmp_instrument *xxi = get_instrument(ctx, xc->ins);
 
-		xc->period = note_to_period(xc->key + sub->xpo +
-			xxi->map[xc->key_porta].xpo, xc->finetune,
-			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+		xc->period = note_to_period(ctx, xc->key + sub->xpo +
+			xxi->map[xc->key_porta].xpo, xc->finetune, xc->per_adj);
 	}
 }
 
@@ -949,6 +947,8 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	int use_ins_vol;
 	int sample_mode;
 	int toneporta_offset;
+	int disabled_toneporta;
+	int retrig_ins;
 	struct xmp_event ev;
 
 	memcpy(&ev, e, sizeof (struct xmp_event));
@@ -961,14 +961,6 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		xc->delayed_ins = 0;
 	}
 
-	/* Keyoff + instrument retrigs current instrument in old fx mode */
-	if (HAS_QUIRK(QUIRK_ITOLDFX)) {
-		if (ev.note == XMP_KEY_OFF && IS_VALID_INSTRUMENT(ev.ins -1)) {
-			ev.note = xc->key + 1;
-			ev.ins = xc->ins + 1;
-		}
-	}
-
 	xc->flags = 0;
 	note = -1;
 	key = ev.note;
@@ -982,6 +974,15 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	candidate_ins = xc->ins;
 	sample_mode = !HAS_QUIRK(QUIRK_VIRTUAL);
 	toneporta_offset = 0;
+	disabled_toneporta = 0;
+	retrig_ins = 0;
+
+	/* Keyoff + instrument retrigs current instrument in old fx mode */
+	if (HAS_QUIRK(QUIRK_ITOLDFX)) {
+		if (ev.note == XMP_KEY_OFF && IS_VALID_INSTRUMENT(ev.ins -1)) {
+			retrig_ins = 1;
+		}
+	}
 
 	/* Notes with unmapped instruments are ignored */
 	if (ev.ins) {
@@ -1017,6 +1018,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	/* Off-Porta.it */
 	if (is_toneporta && ev.fxt == FX_OFFSET) {
+		disabled_toneporta = 1;
 		is_toneporta = 0;
  		if (!HAS_QUIRK(QUIRK_PRENV)) {
 			toneporta_offset = 1;
@@ -1141,12 +1143,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			use_ins_vol = 0;
 		} else {
 			/* portamento_after_keyoff.it test case */
-			reset_env = 1;
+			/* also see suburban_streets o13 c45 */
+			if (ev.ins || !is_toneporta) {
+				if (!disabled_toneporta) {
+					reset_env = 1;
+				}
+			}
 
 			if (is_toneporta) {
-				/* Always retrig on tone portamento: Fix
-				 * portamento in 7spirits.s3m, mod.Biomechanoid
-			 	 */
 				if (not_same_ins || TEST_NOTE(NOTE_END)) {
 					SET(NEW_INS);
 					RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
@@ -1222,7 +1226,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	}
 
 	/* Do after virtual channel copy */
-	if (is_toneporta) {
+	if (is_toneporta || retrig_ins) {
 		if (HAS_QUIRK(QUIRK_PRENV) && ev.ins) {
 			reset_envelopes_carry(ctx, xc);
 		}
@@ -1258,6 +1262,13 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		xc->fadeout = 0x10000;
 	}
 
+	/* See OpenMPT wnoteoff.it vs noteoff3.it */
+	if (retrig_ins && not_same_ins) {
+		SET(NEW_INS);
+		virt_voicepos(ctx, chn, 0);
+		xc->fadeout = 0x10000;
+		RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
+	}
 
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
 
