@@ -24,7 +24,7 @@ void	virt_pastnote		(struct context_data *, int, int);
 void	virt_setvol		(struct context_data *, int, int);
 void	virt_voicepos		(struct context_data *, int, int);
 int	virt_getvoicepos	(struct context_data *, int);
-void	virt_setbend		(struct context_data *, int, int);
+void	virt_setperiod		(struct context_data *, int, double);
 void	virt_setpan		(struct context_data *, int, int);
 void	virt_seteffect		(struct context_data *, int, int, int);
 int	virt_cstat		(struct context_data *, int);
@@ -32,6 +32,7 @@ int	virt_mapchannel		(struct context_data *, int);
 void	virt_resetchannel	(struct context_data *, int);
 void	virt_resetvoice		(struct context_data *, int, int);
 void	virt_reset		(struct context_data *);
+void	virt_release		(struct context_data *, int, int);
 int	virt_getroot		(struct context_data *, int);
 
 #endif /* LIBXMP_VIRTUAL_H */
